# Generated by Django 4.2.1 on 2023-05-15 16:51

from django.db import migrations, models
import django.db.models.deletion
import pulpcore.app.util


class Migration(migrations.Migration):

    dependencies = [
        ('core', '0106_alter_artifactdistribution_distribution_ptr_and_more'),
        ('rpm', '0050_alter_addon_pulp_id_alter_checksum_pulp_id_and_more'),
    ]

    operations = [
        migrations.AlterUniqueTogether(
            name='distributiontree',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='modulemd',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='modulemdobsolete',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='package',
            unique_together=set(),
        ),
        migrations.AlterUniqueTogether(
            name='repometadatafile',
            unique_together=set(),
        ),
        migrations.AddField(
            model_name='distributiontree',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='modulemd',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='modulemddefaults',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='modulemdobsolete',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='package',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='packagecategory',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='packageenvironment',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='packagegroup',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='packagelangpacks',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='repometadatafile',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AddField(
            model_name='updaterecord',
            name='_pulp_domain',
            field=models.ForeignKey(default=pulpcore.app.util.get_domain_pk, on_delete=django.db.models.deletion.PROTECT, to='core.domain'),
        ),
        migrations.AlterField(
            model_name='modulemddefaults',
            name='digest',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='package',
            name='pkgId',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='packagecategory',
            name='digest',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='packageenvironment',
            name='digest',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='packagegroup',
            name='digest',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='packagelangpacks',
            name='digest',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterField(
            model_name='updaterecord',
            name='digest',
            field=models.TextField(db_index=True),
        ),
        migrations.AlterUniqueTogether(
            name='distributiontree',
            unique_together={('_pulp_domain', 'digest')},
        ),
        migrations.AlterUniqueTogether(
            name='modulemd',
            unique_together={('_pulp_domain', 'name', 'stream', 'version', 'context', 'arch')},
        ),
        migrations.AlterUniqueTogether(
            name='modulemddefaults',
            unique_together={('_pulp_domain', 'digest')},
        ),
        migrations.AlterUniqueTogether(
            name='modulemdobsolete',
            unique_together={('_pulp_domain', 'modified', 'module_name', 'module_stream')},
        ),
        migrations.AlterUniqueTogether(
            name='package',
            unique_together={('_pulp_domain', 'name', 'epoch', 'version', 'release', 'arch', 'checksum_type', 'pkgId')},
        ),
        migrations.AlterUniqueTogether(
            name='packagecategory',
            unique_together={('_pulp_domain', 'digest')},
        ),
        migrations.AlterUniqueTogether(
            name='packageenvironment',
            unique_together={('_pulp_domain', 'digest')},
        ),
        migrations.AlterUniqueTogether(
            name='packagegroup',
            unique_together={('_pulp_domain', 'digest')},
        ),
        migrations.AlterUniqueTogether(
            name='packagelangpacks',
            unique_together={('_pulp_domain', 'digest')},
        ),
        migrations.AlterUniqueTogether(
            name='repometadatafile',
            unique_together={('_pulp_domain', 'data_type', 'checksum', 'relative_path')},
        ),
        migrations.AlterUniqueTogether(
            name='updaterecord',
            unique_together={('_pulp_domain', 'digest')},
        ),
    ]
