# Generated by Django 3.2.13 on 2022-04-21 15:14

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('rpm', '0045_modulemd_fields'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='rpmalternatecontentsource',
            options={'default_related_name': '%(app_label)s_%(model_name)s', 'permissions': [('refresh_rpmalternatecontentsource', 'Refresh an Alternate Content Source'), ('manage_roles_rpmalternatecontentsource', 'Can manage roles on ACS')]},
        ),
        migrations.AlterModelOptions(
            name='rpmdistribution',
            options={'default_related_name': '%(app_label)s_%(model_name)s', 'permissions': [('manage_roles_rpmdistribution', 'Can manage roles on an RPM distribution')]},
        ),
        migrations.AlterModelOptions(
            name='rpmpublication',
            options={'default_related_name': '%(app_label)s_%(model_name)s', 'permissions': [('manage_roles_rpmpublication', 'Can manage roles on an RPM publication')]},
        ),
        migrations.AlterModelOptions(
            name='rpmremote',
            options={'default_related_name': '%(app_label)s_%(model_name)s', 'permissions': [('manage_roles_rpmremote', 'Can manage roles on an RPM remotes')]},
        ),
        migrations.AlterModelOptions(
            name='rpmrepository',
            options={'default_related_name': '%(app_label)s_%(model_name)s', 'permissions': [('manage_roles_rpmrepository', 'Can manage roles on RPM repositories'), ('modify_content_rpmrepository', 'Add content to, or remove content from a repository'), ('repair_rpmrepository', 'Copy a repository'), ('sync_rpmrepository', 'Sync a repository'), ('delete_rpmrepository_version', 'Delete a repository version')]},
        ),
        migrations.AlterModelOptions(
            name='ulnremote',
            options={'default_related_name': '%(app_label)s_%(model_name)s', 'permissions': [('manage_roles_ulnremote', 'Can manage roles on an ULN remotes')]},
        ),
    ]
